@@ -3,7 +3,7 @@ package CohortExplorer::Datasource;
 use strict;
 use warnings;
 
-our $VERSION = 0.03;
+our $VERSION = 0.08;
 
 use Carp;
 use Config::General;
@@ -177,6 +177,8 @@ sub set_entity_parameters {
 	$struct->{-columns} =
 	  [ $struct->{-columns}{entity_id}, $struct->{-columns}{visit} || 'NULL' ];
 
+        $struct->{-where}{'ve.identifier'} =  
+
 	eval { ( $stmt, @bind ) = $datasource->sqla()->select(%$struct); };
 
 	if ( catch my $e ) {
@@ -199,13 +201,14 @@ sub set_entity_parameters {
 	}
 
 	# Validate visit_max, only applicable to longitudinal datasources
-	if ( $datasource->type() eq 'longitudinal'
-		&& ( !$datasource->{visit_max} || $datasource->{visit_max} <= 1 ) )
-	{
-		throw_app_hook_exception(
-			error => "Expecting visit (max) > 1 for a longitudinal datasource "
-			  . $datasource->name() );
-	}
+	if (     $datasource->type() eq 'longitudinal' && 
+             (  !$datasource->{visit_max} 
+              || $datasource->{visit_max} <= 1 
+              || $datasource->{visit_max} > 20 
+             )
+           ) {
+		 throw_app_hook_exception( error => "Expecting visit (max) between 1-20 for a longitudinal datasource" );
+	     }
 
 }
 
@@ -393,7 +396,7 @@ CohortExplorer::Datasource - CohortExplorer datasource superclass
 
 =head1 SYNOPSIS
 
-    # The code below shows methods your datasource class overrides;
+    # The code below shows methods your datasource class overrides
 
     package CohortExplorer::Application::REDCap::Datasource;
     use base qw( CohortExplorer::Datasource );
@@ -498,7 +501,7 @@ CohortExplorer::Datasource - CohortExplorer datasource superclass
 
 =head2 initialise( $opts, $config_file )
 
-CohortExplorer::Datasource is an abstract factory; C<initialise()> is the factory method that constructs and returns an object of the datasource supplied as an application option. This class reads the datasource configuration from the config file (i.e. C</etc/CohortExplorer/datasource-config.properties>) to instantiate the datasource object. The config file takes the format below,
+CohortExplorer::Datasource is an abstract factory; C<initialise()> is the factory method that constructs and returns an object of the datasource supplied as an application option. This class reads the datasource configuration from the config file (i.e. C<datasource-config.properties>) to instantiate the datasource object. The config file takes the format below,
 
         <datasource Clinical> 
          namespace=Opal
@@ -538,7 +541,7 @@ Basic constructor.
 
 =head1 PROCESSING
 
-After instantiating the datasource object, the class first calls L<authenticate|/authenticate( $opts )> to perform the user authentication. If the authentication is successful (i.e. a defined <$response>), it sets the default parameters, if any ( via L<default_parameters|/default_parameters( $opts, $response )>). The subsequent steps include calling the methods, L<entity_structure|/entity_structure()>, L<table_structure|/table_structure()>, L<variable_structure|/variable_structure()>, L<datatype_map|/datatype_map()> and validating the return from each method. Upon successful validation the class attempts to set entity, table and variable specific parameters by invoking the methods below:
+After instantiating the datasource object, the class first calls L<authenticate|/authenticate( $opts )> to perform the user authentication. If the authentication is successful (i.e. a defined C<$response>), it sets the default parameters, if any ( via L<default_parameters|/default_parameters( $opts, $response )>). The subsequent steps include calling the methods, L<entity_structure|/entity_structure()>, L<table_structure|/table_structure()>, L<variable_structure|/variable_structure()>, L<datatype_map|/datatype_map()> and validating the return from each method. Upon successful validation the class attempts to set entity, table and variable specific parameters by invoking the methods below:
 
 =head2 set_entity_parameters( $struct )
 
@@ -713,7 +716,7 @@ The method C<execute> from L<DBI> fails to execute the SQL query.
 
 =head1 DEPENDENCIES
 
-Carp
+L<Carp>
 
 L<CLI::Framework::Exceptions>
 
