@@ -3,9 +3,10 @@ package CohortExplorer::Command::Query::Compare;
 use strict;
 use warnings;
 
-our $VERSION = 0.01;
+our $VERSION = 0.08;
 
 use base qw(CohortExplorer::Command::Query);
+use CLI::Framework::Exceptions qw( :all );
 
 #-------
 
@@ -13,38 +14,48 @@ sub usage_text {    # Command is only available to longitudinal datasources
 
 	q\
                 compare [--out|o=<directory>] [--export|e=<table>] [--export-all|a] [--save-command|s] [--stats|S] [--cond|c=<cond>]
-                [variable] : compare entities across visits with/without condition(s) on variable(s)
+                [variable] : compare entities across visits with/without conditions on variables
 
 
                 NOTES
-                   The variables Entity_ID and Visit (if applicable) must not be provided as args as they are already part of 
+                   The variables Entity_ID and Visit (if applicable) must not be provided as arguments as they are already part of 
                    the query-set however, both can be used to impose conditions.
 
-                   Other variables in args/cond (option) must be referenced as 'Table.Variable' or 'Visit.Table.Variable' where
+                   Other variables in arguments/cond (option) must be referenced as 'Table.Variable' or 'Visit.Table.Variable' where
                    visit can be Vany, Vlast, V1, V2, V3, etc.
 
-                   The directory specified within the 'out' option must have RWX enabled for CohortExplorer.
+                   The directory specified within the 'out' option must have RWX enabled (i.e. chmod 777) for CohortExplorer.
 
-                   Conditions can be used using the operators: =, !=, >=, >, <, <=, between, not_between, like, not_like, in, not_in 
-                   and regexp.
+                   Conditions can be imposed using the operators: =, !=, >, <, >=, <=, between, not_between, like, not_like, in, not_in, 
+                   regexp and not_regexp.
 
-                   When a condition is imposed on a regular variable (i.e. not visit) from dynamic tables then it is assumed that the
-                   condition applies to all visits of that variable.
+                   When conditions are imposed on variables with no prefix (e.g. V1, V2, Vany, Vlast etc) it is assumed that the
+                   conditions apply to all visits of those variables.
 
 
                 EXAMPLES
-                   compare --out=/home/user/exports --stats --save-search --cond=V1.CER.Score="{'>','20'}" 
-                           --cond=Vlast.CER.Score="{'<=','30'}" V1.SC.Date
+                   compare --out=/home/user/exports --stats --save-command --cond=V1.CER.Score="{'>','20'}" V1.SC.Date
 
-                   compare --out=/home/user/exports --export=CER --cond=SD.Subject_Sex="{'=','Male'}" V1.CER.Score V1.DIS.Status
+                   compare --out=/home/user/exports --export=CER --cond=SD.Subject_Sex="{'=','Male'}" V1.CER.Score V3.DIS.Status
 
-                   compare -o'/home/user/exports' -Ssa -c Vlast.CER.Score="{'in',['25','30','40']}" DIS.Status 
+                   compare -o /home/user/exports -Ssa -c Vlast.CER.Score="{'in',['25','30','40']}" DIS.Status 
 
-                   compare -o'/home/user/exports' -e CER -e SD -c Vlast.CER.Score="{'between',['25','30']}" DIS.Status
+                   compare -o /home/user/exports -e CER -e SD -c Vlast.CER.Score="{'between',['25','30']}" DIS.Status
 
              \;
 }
 
+
+sub get_validation_variables {
+
+        my ( $self ) = @_;
+
+        my $datasource = $self->cache->get('cache')->{datasource};
+
+        return [ 'Entity_ID', keys %{ $datasource->variables() }, @{ $datasource->visit_variables() } ];  
+
+}
+
 sub get_query_parameters {
 
 	my ( $self, $opts, $datasource, @args ) = @_;
@@ -54,32 +65,29 @@ sub get_query_parameters {
 	my $struct        = $datasource->entity_structure();
 	my %param;
 
-    # Extract all variables from args/cond (option) except Entity_ID and Visit as they are dealt separately
-	my @vars = grep( !/^Entity_ID|Visit$/,
-		keys %{
-			{
-				map { $_ => 1 } map { s/^V(any|last|[0-9]+)\.//; $_ } @args,
-				keys %{ $opts->{cond} }
-			}
-		  } );
+        # Extract all variables from args/cond (option) except Entity_ID and Visit as they are dealt separately
+	my @vars = grep( !/^(Entity_ID|Visit)$/,
+		         keys % {
+			         {
+				   map { $_ => 1 } map { s/^V(any|last|[0-9]+)\.//; $_ } @args, keys %{ $opts->{cond} }
+			         }
+		         } 
+                       );
 
 	for my $var (@vars) {
 		$var =~ /^([^\.]+)\.(.+)$/; # Extract tables and variable names, a variable is referenced as 'Table.Variable'
 		# Build a hash with keys 'static' and 'dynamic'
 		# Each keys contains its own sql parameters
 		my $table_type = grep ( /^$1$/, @static_tables ) ? 'static' : 'dynamic';
-		push
-		  @{ $param{$table_type}{-where}{ $struct->{-columns}{table} }{-in} },
-		  $1;
-		push @{ $param{$table_type}{-where}{ $struct->{-columns}{variable} }
-			  {-in} }, $2;
+		push @{ $param{$table_type}{-where}{ $struct->{-columns}{table} }{-in} }, $1;
+		push @{ $param{$table_type}{-where}{ $struct->{-columns}{variable} }{-in} }, $2;
 
 		if ( $table_type eq 'dynamic' ) {
 
 			# Each column corresponds to one visit
 			for (@visits) {
 				push @{ $param{$table_type}{-columns} },
-" CAST( GROUP_CONCAT( IF( CONCAT( $struct->{-columns}{table}, '.', $struct->{-columns}{variable} ) = '$var'"
+                                   " CAST( GROUP_CONCAT( IF( CONCAT( $struct->{-columns}{table}, '.', $struct->{-columns}{variable} ) = '$var'"
 				  . " AND $struct->{-columns}{visit} = $_, $struct->{-columns}{value}, NULL)) AS "
 				  . ( uc $variables->{$var}{type} )
 				  . " ) AS `V$_.$var`";
@@ -100,20 +108,16 @@ sub get_query_parameters {
 
 		else {
 
-	        # Build conditions for visit variables e.g. V1.Var, Vlast.Var, Vany.Var etc.
-	        # Values inside array references are joined as 'OR' and hashes as 'AND'
-			my @visit_vars = grep( /^(V(any|last|[0-9]+)\.$var|$var)$/,
-				keys %{ $opts->{cond} } );
+	                # Build conditions for visit variables e.g. V1.Var, Vlast.Var, Vany.Var etc.
+	                # Values inside array references are joined as 'OR' and hashes as 'AND'
+			my @visit_vars = grep( /^(V(any|last|[0-9]+)\.$var|$var)$/, keys %{ $opts->{cond} } );
 
 			for my $visit_var ( sort @visit_vars ) {
-
-                # Last visits (i.e. Vlast) for entities are not known in advance so practically any
-                # visit can be the last visit for any entity
+                                # Last visits (i.e. Vlast) for entities are not known in advance so practically any
+                                # visit can be the last visit for any entity
 				if ( $visit_var =~ /^Vlast\.$var$/ ) {
-					my ( $opr, $val ) =
-					  ( $opts->{cond}{"Vlast.$var"} =~
-						  /^\{\'([^\']+)\',(.+)\}$/ );
-					$val = !$2 ? undef : eval $2;
+				     my ( $opr, $val ) = ( $opts->{cond}{"Vlast.$var"} =~ /^\{\'([^\']+)\',(.+)\}$/ );
+					  $val = !$2 ? undef : eval $2;
 
 					if ( defined $param{$table_type}{-having}{-or} ) {
 						map {
@@ -136,9 +140,7 @@ sub get_query_parameters {
 
 				# Vany includes all visit variables joined as 'OR'
 				elsif ( $visit_var =~ /^Vany\.$var$/ ) {
-					my ( $opr, $val ) =
-					  ( $opts->{cond}{"Vany.$var"} =~
-						  /^\{\'([^\']+)\',(.+)\}$/ );
+					my ( $opr, $val ) = ( $opts->{cond}{"Vany.$var"} =~ /^\{\'([^\']+)\',(.+)\}$/ );
 					$val = !$2 ? undef : eval $2;
 
 					if ( defined $param{$table_type}{-having}{-and} ) {
@@ -173,8 +175,8 @@ sub get_query_parameters {
 					}
 				}
 
-                # When a condition is imposed on a variable (with no prefix V1, Vlast, Vany) from dynamic table(s) then
-                # it is assumed that the condition applies to all visits of that variable (i.e. 'AND' case)
+                                # When a condition is imposed on a variable (with no prefix V1, Vlast, Vany)
+                                # assume the condition applies to all visits of that variable (i.e. 'AND' case)
 				else {
 
 					map {
@@ -197,7 +199,7 @@ sub get_query_parameters {
 
 		else {
 
-		    # Entity_ID and Visit are added to the list of SQL cols in dynamic param
+		        # Entity_ID and Visit are added to the list of SQL cols in dynamic param
 			unshift @{ $param{$_}{-columns} },
 			  (
 				$struct->{-columns}{entity_id} . '|`Entity_ID`',
@@ -234,14 +236,15 @@ sub get_query_parameters {
 	return \%param;
 }
 
+
 sub process_result_set {
 
 	my ( $self, $opts, $datasource, $result_set, $dir, $csv, @args ) = @_;
 
-    # Header of the csv output must pay attention to args and variables on which the condition is imposed
-    # Extract visit specific variables from the result-set based on the variables provided as args/cond (option).
-    # For example, variables in args/cond variables are V1.Var and Vlast.Var but as the result-set contains all visits of
-    # the variable 'var' so discard V2.var and V3.var and select V1.var and the equivalent Vlast.Var
+        # Header of the csv output must pay attention to args and variables on which the condition is imposed
+        # Extract visit specific variables from the result-set based on the variables provided as args/cond (option).
+        # For example, variables in args/cond variables are V1.Var and Vlast.Var but as the result-set contains all visits of
+        # the variable 'var' so discard V2.var and V3.var and select V1.var and the equivalent Vlast.Var
 	my $index = $result_set->[0][3] && $result_set->[0][3] eq 'Visit' ? 3 : 0;
 
 	# Compiling regex to extract variables specified as args/cond (option)
@@ -249,63 +252,58 @@ sub process_result_set {
 
 	$regex = qr/$regex/;
 
-	my @index_to_use = sort { $a <=> $b } keys %{
+	my @index_to_use = sort { $a <=> $b } keys % {
 		{
 			map { $_ => 1 } (
-				0 .. $index,
-				grep( $result_set->[0][$_] =~ $regex,
-					0 .. $#{ $result_set->[0] } )
-			)
+				          0 .. $index,
+				          grep( $result_set->[0][$_] =~ $regex, 0 .. $#{ $result_set->[0] } )
+			                )
 		}
 	  };
 
 	my @vars = @{ $result_set->[0] }[@index_to_use];
 
-    # Extract last visit specific variables (i.e. Vlast.Var) in args/cond (option)
-	my @last_visit_vars =
-	  grep( /^Vlast\./, ( @args, keys %{ $opts->{cond} } ) );
+        # Extract last visit specific variables (i.e. Vlast.Var) in args/cond (option)
+	my @last_visit_vars = keys % { { map { $_ => 1 } grep( /^Vlast\./, ( @args, keys %{ $opts->{cond} } ) ) }  };  
 
-	# The entities output by the query are stored within a list
+	# Entities from the query are stored within a list
 	my @result_entity;
 
-	my $fh = FileHandle->new("> $dir/QueryOutput.csv")
-	  or throw_cmd_run_exception( error => "Failed to open file: $!" );
-	$csv->combine( ( @vars, @last_visit_vars ) )
-	  ? print $fh $csv->string() . "\n"
-	  : throw_cmd_run_exception( error => $csv->error_input() );
+        my $file = File::Spec->catfile($dir, "QueryOutput.csv");
+
+	my $fh = FileHandle->new("> $file") or throw_cmd_run_exception( error => "Failed to open file: $!" );
+        my @cols = ( @vars, @last_visit_vars );	        
+           $csv->print($fh, \@cols) or throw_cmd_run_exception( error => $csv->error_diag() );
 
 	for my $row ( 1 .. $#$result_set ) {
 		push @result_entity, $result_set->[$row][0];
 
 		# Sort visits in the Visit column
-		$result_set->[$row][3] =
-		  join( ',', ( sort { $a <=> $b } split ',', $result_set->[$row][3] ) )
-		  if ( $index == 3 );
-		my @last_visit_cols =
-		  map { s/^Vlast\.//; "V$result_set->[$row][2].$_" } @last_visit_vars;
+		$result_set->[$row][3] = join( ', ', 
+                                               ( sort { $a <=> $b } split ',', $result_set->[$row][3] ) 
+                                             )  if ( $index == 3 );
+
+		my @last_visit_cols = map { s/^Vlast\.//; "V$result_set->[$row][2].$_" } @last_visit_vars;
+
 		my @last_visit_vals;
 
-		for my $last_visit_col (@last_visit_cols) {
-			my ($index) =
-			  grep { $result_set->[0][$_] eq $last_visit_col }
-			  0 .. $#{ $result_set->[0] };
+		for my $col (@last_visit_cols) {
+			my ($index) = grep { $result_set->[0][$_] eq $col } 0 .. $#{ $result_set->[0] };
 			push @last_visit_vals, $result_set->[$row][$index];
 		}
-		$csv->combine( ( map { $result_set->[$row][$_] } @index_to_use ),
-			@last_visit_vals )
-		  ? print $fh $csv->string()
-		  . "\n"
-		  : throw_cmd_run_exception( error => $csv->error_input() );
+                
+                my @vals = ( ( map { $result_set->[$row][$_] } @index_to_use ), @last_visit_vals );
+	        $csv->print($fh, \@vals) or throw_cmd_run_exception( error => $csv->error_diag() );
 	}
 
 	$fh->close();
 	return \@result_entity;
 }
 
+
 sub process_table {
 
-	my ( $self, $table, $datasource, $table_data, $dir, $csv, $result_entity ) =
-	  @_;
+	my ( $self, $table, $datasource, $table_data, $dir, $csv, $result_entity ) = @_;
 
 	my @static_tables = @{ $datasource->static_tables() || [] };
 	my $table_type =
@@ -343,31 +341,29 @@ sub process_table {
 	}
 
 	# Write table data
-	my $file      = "$dir/$table.csv";
+	my $file      = File::Spec->catfile($dir, "$table.csv");
 	my $untainted = $1 if ( $file =~ /^(.+)$/ );
-	my $fh        = FileHandle->new("> $untainted")
-	  or throw_cmd_run_exception( error => "Failed to open file: $!" );
-	$csv->combine( ( qw(Entity_ID), @header ) )
-	  ? print $fh $csv->string() . "\n"
-	  : throw_cmd_run_exception( error => $csv->error_input() );
+	my $fh        = FileHandle->new("> $untainted") or throw_cmd_run_exception( error => "Failed to open file: $!" );
+        my @cols       = ( qw(Entity_ID), @header );
+           $csv->print($fh, \@cols) or throw_cmd_run_exception( error => $csv->error_diag() );
 
 	# Write data for entities present in the result set
 	for my $entity (@$result_entity) {
-		$csv->combine( $entity, map { $data{$entity}{$_} } @header )
-		  ? print $fh $csv->string() . "\n"
-		  : throw_cmd_run_exception( error => $csv->error_input() );
+            my @vals = ( $entity, map { $data{$entity}{$_} } @header );
+	       $csv->print($fh, \@vals) or throw_cmd_run_exception( error => $csv->error_diag() );
 	}
 
 	$fh->close();
 }
 
+
 sub get_stats_data {
 
 	my ( $self, $result_set ) = @_;
 	my $index = $result_set->[0][3] && $result_set->[0][3] eq 'Visit' ? 3 : 0;
 	my %data;
 
-    # Remove visit suffix Vany, Vlast, V1, V2 etc. from the variables in the result-set (i.e. args/cond (option))
+        # Remove visit suffix Vany, Vlast, V1, V2 etc. from the variables in the result-set (i.e. args/cond (option))
 	my @vars = keys %{
 		{
 			map { s/^V(any|last|[0-9]+)\.//; $_ => 1 }
@@ -380,9 +376,7 @@ sub get_stats_data {
 		for my $var (@vars) {
 			$data{ $result_set->[$row][0] }{$var} = [
 				map {
-					    $result_set->[0][$_] =~ /$var$/
-					  ? $result_set->[$row][$_] || ()
-					  : ()
+				       $result_set->[0][$_] =~ /$var$/ ? $result_set->[$row][$_] || () : ()
 				  } $index + 1 .. $#{ $result_set->[0] }
 			];
 			$data{ $result_set->[$row][0] }{'Visit'} =
@@ -404,7 +398,7 @@ __END__
 
 =head1 NAME
 
-CohortExplorer::Command::Query::Compare - CohortExplorer command to compare entities across visits with or without condition(s) on variable(s)
+CohortExplorer::Command::Query::Compare - CohortExplorer class to compare entities across visits
 
 =head1 SYNOPSIS
 
@@ -418,21 +412,29 @@ The compare command enables the user to compare entities across visits. The user
 
 This class is inherited from L<CohortExplorer::Command::Query> and overrides the following methods:
 
+=head2 usage_text()
+
+This method returns the usage information for the command.
+
+=head2 get_validation_variables()
+
+This method returns a ref to the list containing Entity_ID, all visit and non visit variables for validating arguments and condition option(s).
+
 =head2 get_query_parameters( $opts, $datasource, @args )
 
 This method returns a hash ref with keys, C<static>, C<dynamic> or C<both> depending on the variables supplied within arguments and conditions. The value of each key is a hash containing SQL parameters, C<-columns>, C<-from>, C<-where>, C<-group_by> and C<-having>.
 
 =head2 process_result_set( $opts, $datasource, $result_set, $dir, $csv, @args )
      
-This method writes result set to csv file and return a ref to a list containing entity_ids.
+This method writes result set to csv file and return a ref to the list containing C<Entity_IDs>.
         
 =head2 process_table( $table, $datasource, $table_data, $dir, $csv, $result_entity )
         
-This method writes the table data into a csv file for entities present in the result set. For static tables the csv contains C<Entity_ID> followed by variables' values and in case of dynamic tables the csv contains C<Entity_ID> followed by values of all visit variables.
+This method writes the table data into a csv file for entities present in the result set. For static tables the csv contains C<Entity_ID> followed by variables' values and in case of dynamic tables the csv contains C<Entity_ID> followed by the values of all visit variables.
 
 =head2 get_stats_data( $result_set )
 
-This method returns a hash ref with entity_id as keys and variable-value hash as its value. The statistics in this command are computed with respect to the entity_id and number of observation for each variable is equal to the number of times/visits each variable was recorded.
+This method returns a hash ref with C<Entity_ID> as keys and variable-value pairs as its value. The statistics in this command are computed with respect to the C<Entity_ID> and number of observation for each variable is equal to the number of times/visits each variable was recorded.
  
 =head1 OPTIONS
 
@@ -456,36 +458,38 @@ Save command
 
 =item B<-S>, B<--stats>
 
-Show statistics
+Show summary statistics
 
 =item B<-c> I<COND>, B<--cond>=I<COND>
             
-Impose conditions using the operators: =, !=, >=, >, <, <=, between, not_between, like, not_like, in, not_in and regexp.
+Impose conditions using the operators: C<=>, C<!=>, C<E<gt>>, C<E<lt>>, C<E<gt>=>, C<E<lt>=>, C<between>, C<not_between>, C<like>, C<not_like>, C<in>, C<not_in>, C<regexp> and C<not_regexp>.
 
 =back
 
 =head1 NOTES
 
-The variables C<Entity_ID> and C<Visit> (if applicable) must not be provided as arguments as they are already part of the query-set. However, the user has the liberty to impose conditions on both the C<Entity_ID> and C<Visit>, using the condition option. Other variables in arguments and conditions must be referenced as C<Table.Variable> or C<Visit.Table.Variable> where Visit = Vlast, Vany, V1, V2, etc. When a condition is imposed on variable(s) (i.e. with no prefix V1, V2, Vany or Vlast) from dynamic table(s) then it is assumed that the condition applies to all visits of the variable(s). The directory specified within the C<out> option must have RWX enabled for CohortExplorer.
+The variables C<Entity_ID> and C<Visit> (if applicable) must not be provided as arguments as they are already part of the query-set. However, the user has the liberty to impose conditions on both the C<Entity_ID> and C<Visit>, using the C<cond> option. Other variables in arguments and conditions must be referenced as C<Table.Variable> or C<Visit.Table.Variable> where Visit = C<V1>, C<V2>, C<Vany>, C<Vlast> etc. When a condition is imposed on variables with no prefix C<V1>, C<V2>, C<Vany> or C<Vlast>, it is assumed that the condition applies to all visits of those variables. The directory specified within the C<out> option must have RWX enabled for CohortExplorer.
 
 =head1 EXAMPLES
 
-compare --out=/home/user/exports --stats --save-search --cond=V1.CER.Score="{'>','20'}" --cond=Vlast.CER.Score="{'<=','30'}" V1.SC.Date
+ compare --out=/home/user/exports --stats --save-command --cond=V1.CER.Score="{'>','20'}" V1.SC.Date
 
-compare --out=/home/user/exports --export=CER --cond=SD.Subject_Sex="{'=','Male'}" V1.CER.Score V1.DIS.Status
+ compare --out=/home/user/exports --export=CER --cond=SD.Subject_Sex="{'=','Male'}" V1.CER.Score V3.DIS.Status
 
-compare -o'/home/user/exports' -Ssa -c Vlast.CER.Score="{'in',['25','30','40']}" DIS.Status 
+ compare -o /home/user/exports -Ssa -c Vlast.CER.Score="{'in',['25','30','40']}" DIS.Status 
 
-compare -o'/home/user/exports' -e CER -e SD -c Vlast.CER.Score="{'between',['25','30']}" DIS.Status
+ compare -o /home/user/exports -e CER -e SD -c Vlast.CER.Score="{'between',['25','30']}" DIS.Status
 
 =head1 DIAGNOSTICS
 
-This class throws C<throw_cmd_run_exception> exception imported from L<CLI::Framework::Exception> if L<Text::CSV_XS> fails to construct a csv string from list containing variables' values.
+This class throws C<throw_cmd_run_exception> exception imported from L<CLI::Framework::Exceptions> if L<Text::CSV_XS> fails to construct a csv string from the list containing variables' values.
 
 =head1 SEE ALSO
 
 L<CohortExplorer>
 
+L<CohortExplorer::Datasource>
+
 L<CohortExplorer::Command::Describe>
 
 L<CohortExplorer::Command::Find>
